/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2013 digitalSTROM.org, Zurich, Switzerland
 *  Author: Sibylle Kohler
 */

/**
 * opens URL in a new tab 
 * works only for chrome and ie, all other browsers open new window. 
 * depends on browser settings (open new window in tab)
 */
function openLinkInNewTab (sURL) {
    fenster = window.open(sURL, '_newtab');
    fenster.focus();
}

/**
 * opens terms and condition page in a popup window 
 * depends on browser setting (open new window in tab)
 */
function openTermsAndConditions () {
    fenster = window.open(TermsAndConditionsURL,'_new', 'width=900,height=500,resizable=yes');
    fenster.focus();
}

/**
 * Extract parameter value from query string
 * @param name [in] name of the parameter
 * @returns the value of the parameter as string, empty string if parameter not found
 */
function getParameterByName(name)
{
    name = name.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
    var regexS = "[\\?&]" + name + "=([^&#]*)";
    var regex = new RegExp(regexS);
    var results = regex.exec(window.location.search);
    if(results === null) {
        return "";
    } else {
        return decodeURIComponent(results[1].replace(/\+/g, " "));
    }
}

/**
 * Removes unnecessary query string parts from callback url
 * @returns callbackURL as string
 */
function prepareCallbackURL()
{
    var queryStringPos = document.location.href.indexOf('?');
    var callbackURL = document.location.href;
    if (-1 != queryStringPos) {
        callbackURL = document.location.href.substring(0, queryStringPos);
    }
    if ('expert' === getParameterByName('mode')) {
        callbackURL =  callbackURL + '?mode=expert';
    }
    return callbackURL;
}

/**
 * Read out MAC address of dSS and format url for registration.
 * @returns {String} url to redirect to for registration
 */
function buildRegistrationURL()
{
    var sURL = registrationURL;
    var sMAC = getPropertyString('/system/host/interfaces/eth0/mac');
    var sLang = dss.staticDataModel.activeLanguage.split('_');
    sURL = sURL + '&mac=' + sMAC;
    sURL = sURL + '&lang=' +sLang[0];
    sURL = sURL + '&callbackURL=' + prepareCallbackURL();
    return sURL;
}

/**
 * set a property value in property tree.
 * If the node does not exists yet it will be created.
 * @param sProp [in] property path
 * @param sValue [in] value of the property, allowed number, boolean and string
*/
function setProperty(sProp,sValue)
{
    var sURL='';
    var sPfad=escape(sProp);
    	
    if (typeof(sValue)=='number')
    {
        sURL='/json/property/setInteger';// ?path=' + sPfad + '&value=' + (sValue);
    } else { 
        if (typeof(sValue)=='boolean') {
            sURL='/json/property/setBoolean';// ?path=' + sPfad + '&value=' + (sValue);
        } else { 
            sURL='/json/property/setString';// ?path=' + sPfad + '&value=' + (sValue);
        }
    }
    dss.ajaxAssyncRequestWithoutAnswer(sURL,{'path':sPfad,'value': (sValue)});
}

/**
 * set a property archive flag in property tree.
 * @param sProp [in] property path
*/
function setPropertyArchive(sProp)
{
    var sURL='';
    var sPfad=escape(sProp);

    sURL='/json/property/setFlag';
	
    // '/json/property/setFlag?path=' + sPfad + '&value=true&flag=ARCHIVE';

    dss.ajaxAssyncRequestWithoutAnswer(sURL,{'path':sPfad,'value': 'true','flag':'ARCHIVE'});
}

/**
 * get a property string value from property tree.
 * @param sProp [in] property path
 * @return property value, null if not found
*/
function getPropertyString(sProp)
{
    var sURL='/json/property/getString';
    var sPfad=escape(sProp);
    var sReturn=dss.ajaxSyncRequest(sURL,{'path':sPfad});
    var obj=eval('('+sReturn+')');
    if (obj.ok) {
        return obj.result.value;
    } else {
        return null;
    }
}

/**
 * get a property interger value from property tree.
 * @param sProp [in] property path
 * @return property value, null if not found
*/
function getPropertyInteger(sProp)
{
    var sURL='/json/property/getInteger';
    var sPfad=escape(sProp);
    var sReturn=dss.ajaxSyncRequest(sURL,{'path':sPfad});
    var obj=eval('('+sReturn+')');
    if (obj.ok) {
        return obj.result.value;
    } else {
        return null;
    }
}

/**
 * get a property boolean value from property tree.
 * @param sProp [in] property path
 * @return property value, null if not found
*/
function getPropertyBoolean(sProp)
{
    var sURL='/json/property/getBoolean';
    var sPfad=escape(sProp);
    var sReturn=dss.ajaxSyncRequest(sURL,{'path':sPfad});
    var obj=eval('('+sReturn+')');
    if (obj.ok) {
        return obj.result.value;
    } else {
        return null;
    }
}

/**
 * opens a window to send the mail with the temporary pagekite link.
 * @param tmpLink [in] temporary pagekite link
 * @return timePeriod [in] time period or which the link is valid
*/
function sendMail(tmpLink, timePeriod) {
    var sBody = Ext.String.format2(_("with the following link you can access the next {timeperiod} hours on my digitalSTROM installation."), {timeperiod:timePeriod});
    var mailto_link = "mailto:?subject=" + encodeURIComponent(sSubject) + "&body=" + encodeURIComponent(sTitle) + "%0D%0A%0D%0A" + encodeURIComponent(sBody) + "%0D%0A" + "https://" + tmpLink+ "%0D%0A%0D%0A"+ encodeURIComponent(sBody2) +"%0D%0A%0D%0A"+encodeURIComponent(sGreetings)+"%0D%0A" + encodeURIComponent(sFirstName) + "%20" + encodeURIComponent(sLastName);
    window.location = mailto_link;
}

/**
 * opens a window to resend the mail with the temporary pagekite link with the information 
 * of expiration time
 * @param tmpLink [in] temporary pagekite link
 * @param expirationTime [in] time when the link expires
 */
function resendMail(tmpLink, expirationTime) {
    var sBody = Ext.String.format2(_("with the following link you can access my digitalSTROM installation until {expirationtime}."), {expirationtime:expirationTime});
    var mailto_link = "mailto:?subject=" + encodeURIComponent(sSubject) + "&body=" + encodeURIComponent(sTitle) + "%0D%0A%0D%0A" + encodeURIComponent(sBody) + "%0D%0A" + "https://" + tmpLink+ "%0D%0A%0D%0A"+ encodeURIComponent(sBody2) +"%0D%0A%0D%0A"+encodeURIComponent(sGreetings)+"%0D%0A" + encodeURIComponent(sFirstName) + "%20" + encodeURIComponent(sLastName);
    window.location.href = mailto_link;
}

/**
 * build string and show error message
 * @param code
 * @param servicecode
 */
function showErrorMsg(baseMsg, code, serviceCode) {
    var sErrorMsg = baseMsg;
    if (serviceCode && serviceCode != 0) {
        switch (parseInt(serviceCode)) {
        case 1: //No connection to digitalstrom.com
            sErrorMsg = sErrorMsg + '<br/>' + sErrorNetwork;
            break;
        case 4: //User, password, MAC failed
            sErrorMsg = sErrorMsg + '<br/>' + sErrorCredentials;
            break;
        case 6: //Unknown MAC-address
            sErrorMsg = sErrorMsg + '<br/>' + sErrorMAC;
            break;
        case 9: //<dSID> is unknown
            sErrorMsg = sErrorMsg + '<br/>' + sErrordSID;
            break;
        case 46: //Relay Link fails
        case 99: //Unknown Exception
        default: 
            break;
        }
    } else {
        switch (parseInt(code)) {
        case 81:  //web service related error / no connection to web service
            sErrorMsg = sErrorMsg + '<br/>' + sErrorNetwork;
            break;
        case 84: // ERR_DEFAULT_PW
            sErrorMsg = sErrorMsg + '<br/>' + sErrorDefaultPWD;
            break;
        case 82: //error configuring pagekite
        case 83: //error enabling pagekite
        default:              
            break;
        }
    }
    sErrorMsg = sErrorMsg + '<br/>' + sErrorHint;
    RemoteConnectivity.errorDialog.showWindow(sErrorMsg);
}

/**
 * Gets expiration time from property tree
 * @returns expiration time, '' if not existing
 */
function getExpirationTime() {
    var sString = dss.ajaxSyncRequest('/json/property/query', {
        'query' : '/scripts/remote-connectivity/tempAccess(expirationTime)'
    });
    var result = Ext.JSON.decode(sString);
    if ((result.result!=null) && (result.result['tempAccess']!=null)) {
        if (result.result['tempAccess'][0]!=null) {
            return result.result['tempAccess'][0]['expirationTime'];
        }
    }    
    return '';
}

/**
 * formats the expiring time according our prefered format
 * @param expiring expiring time string of date object
 * @returns string representing expiration time
 */
function formatExpirationTime(expiring) {
    var val = new Date(expiring);
    return Ext.Date.format(val, "l, d.m.Y H:i");    
}

/**
 * Gets temp link from property tree
 * @returns temp link, '' if not existing
 */
function getTempLink() {
    var sString = dss.ajaxSyncRequest('/json/property/query', {
        'query' : '/scripts/remote-connectivity/tempAccess(relayLink)'
    });
    var result = Ext.JSON.decode(sString);
    if ((result.result!=null) && (result.result['tempAccess']!=null)) {
        if (result.result['tempAccess'][0]!=null) {        
            return result.result['tempAccess'][0]['relayLink'];
        }
    }    
    return '';
}

/**
 * Check for compatible dSS version.
 * @returns true if compatible or we are on some testing feed, false otherwise
 */
function checkdSSVersion(requiredVersion, installedVersion, installedFeed) {
    if (installedFeed === 'production') {
        var Ausdruck = /\d+\.\d+\.\d+/;
        var sVersion = /\d+\.\d+\.\d+/.exec(installedVersion);
        if (sVersion >= requiredVersion) {
            return true;
        } else {
            return false;    
        }
    } else {
        // we are in any non-production feed i.e. 'devel', 'testing', 'staging-fieldtest','fieldtest', 'exhibition'
        return true;
    }
}

/**
 * Set a timer to refresh the page if temporary access expired.
 */
function installAutoRefresh(expirationTime) {
    var expirationDate = new Date(expirationTime);
    var currentDate = new Date();
    var delayTime = 60*1000; // 1 minute
    var linkValidity = (expirationDate.getTime() - currentDate.getTime()) + delayTime;
    if (linkValidity > 0) {
        autoRefesh = setTimeout('RemoteConnectivity.checkTempAccess()', linkValidity);
    }
}

/**
 * clear timers if page is reloaded or closed.
 */
function cleanUp() {
    clearTimeout(autoRefesh);
    clearTimeout(verificationDelay);
}